import { OnInit } from '@angular/core';
import { AbstractControl, ValidationErrors } from '@angular/forms';
import { Observable } from 'rxjs';
import { FormComponentSuperclass } from './form-component-superclass';
import * as i0 from "@angular/core";
/**
 * Extend this when creating a form component that simply wraps existing ones, to reduce a lot of boilerplate.
 *
 * To wrap a single form control use the subclass {@linkcode WrappedFormControlSuperclass}:
 * ```ts
 * @Component({
 *   template: `<input [formControl]="control">`,
 *   providers: [provideValueAccessor(StringComponent)],
 * })
 * class StringComponent extends WrappedFormControlSuperclass<string> {}
 * ```
 *
 * Example of wrapping multiple inner components:
 * ```ts
 * class FullName {
 *   firstName = '';
 *   lastName = '';
 * }
 *
 * @Component({
 *   selector: 'app-full-name',
 *   template: `
 *     <div [formGroup]="control">
 *       <input id="first" formControlName="firstName" />
 *       <input id="last" formControlName="lastName" />
 *     </div>
 *   `,
 *   providers: [provideValueAccessor(FullNameComponent)],
 * })
 * class FullNameComponent extends WrappedControlSuperclass<FullName> {
 *   control = new FormGroup({
 *     firstName: new FormControl(),
 *     lastName: new FormControl(),
 *   });
 *
 *   protected outerToInnerValue(outer: FullName | null): FullName {
 *     // `outer` can come in as `null` during initialization when the user binds with `ngModel`
 *     return outer || new FullName();
 *   }
 * }
 * ```
 *
 * Example when you need to modify the wrapped control's value:
 * ```ts
 * @Component({
 *   template: `<input type="datetime-local" [control]="formControl">`,
 *   providers: [provideValueAccessor(DateComponent)],
 * })
 * class DateComponent extends WrappedFormControlSuperclass<Date, string> {
 *   protected innerToOuterValue(inner: string): Date {
 *     return new Date(inner + "Z");
 *   }
 *
 *   protected outerToInnerValue(outer: Date): string {
 *     if (outer === null) {
 *       return ""; // happens during initialization
 *     }
 *     return outer.toISOString().substr(0, 16);
 *   }
 * }
 * ```
 *
 * If you bind to your component using an {@linkcode NgControl} (e.g. when using `ngModel`), validation errors will be synchronized between it and the control inside your component. You can override various methods below to control or disable that process. Note that validation, `statuschanges`, and `valuechanges` may all happen more often as a result of this synchronization.
 */
export declare abstract class WrappedControlSuperclass<OuterType, InnerType = OuterType> extends FormComponentSuperclass<OuterType> implements OnInit {
    #private;
    /** Bind this to your inner form control to make all the magic happen. */
    abstract control: AbstractControl;
    constructor();
    ngOnInit(): void;
    /** Called as angular propagates values changes to this `ControlValueAccessor`. You normally do not need to use it. */
    handleIncomingValue(outer: OuterType): void;
    /** Called as angular propagates disabled changes to this `ControlValueAccessor`. You normally do not need to use it. */
    setDisabledState(isDisabled: boolean): void;
    /**
     * Override this to for full control over the stream of values passed in to your subclass.
     *
     * In this example, incoming values are debounced before being passed through to the inner form control
     * ```ts
     * setUpOuterToInnerValue$(outer$: Observable<OuterType>): Observable<InnerType> {
     *   return outer$.pipe(
     *     debounce(300),
     *     map((outer) => doExpensiveTransformToInnerValue(outer)),
     *   );
     * }
     * ```
     *
     * For a simple transformation, see {@linkcode #outerToInnerValue} instead.
     */
    protected setUpOuterToInnerValue$(outer$: Observable<OuterType>): Observable<InnerType>;
    /**
     * Override this to modify a value coming from the outside to the format needed within this component.
     *
     * For more complex needs, see {@linkcode #setUpOuterToInnerValue$} instead.
     */
    protected outerToInnerValue(outer: OuterType): InnerType;
    /**
     * Override this to for full control over the stream of values emitted from your subclass.
     *
     * In this example, illegal values are not emitted
     * ```ts
     * setUpInnerToOuterValue$(inner$: Observable<InnerType>): Observable<OuterType> {
     *   return inner$.pipe(
     *     filter((inner) => isLegalValue(outer)),
     *   );
     * }
     * ```
     *
     * For a simple transformation, see {@linkcode #innerToOuterValue} instead.
     */
    protected setUpInnerToOuterValue$(inner$: Observable<InnerType>): Observable<OuterType>;
    /**
     * Override this to modify a value coming from within this component to the format expected on the outside.
     *
     * For more complex needs, see {@linkcode #setUpInnerToOuterValue$} instead.
     */
    protected innerToOuterValue(inner: InnerType): OuterType;
    /**
     * Override this to for full control over the stream of validation errors synchronized in to your subclass.
     *
     * For a simple transformation, see {@linkcode #outerToInnerErrors} instead.
     */
    protected setUpOuterToInnerErrors$(outer$: Observable<ValidationErrors>): Observable<ValidationErrors>;
    /**
     * Override this to modify validation errors that synchronize in to this component.
     *
     * In this example we assume the `required` validation is handled by the user and should not affect internal validation
     * ```ts
     * protected override outerToInnerErrors(
     *   errors: ValidationErrors,
     * ): ValidationErrors {
     *   return omit(errors, 'required');
     * }
     * ```
     *
     * For more complex needs, see {@linkcode #setUpOuterToInnerErrors$} instead.
     */
    protected outerToInnerErrors(errors: ValidationErrors): ValidationErrors;
    /**
     * Override this to for full control over the stream of validation errors synchronized out from your subclass.
     *
     * In this example, synchronization is turned off:
     * ```ts
     * protected override setUpInnerToOuterErrors$(): Observable<ValidationErrors> {
     *   return EMPTY;
     * }
     * ```
     *
     * For a simple transformation, see {@linkcode #innerToOuterErrors} instead.
     */
    protected setUpInnerToOuterErrors$(inner$: Observable<ValidationErrors>): Observable<ValidationErrors>;
    /**
     * Override this to modify validation errors that synchronize out from this component.
     *
     * For more complex needs, see {@linkcode #setUpInnerToOuterErrors$} instead.
     */
    protected innerToOuterErrors(errors: ValidationErrors): ValidationErrors;
    static ɵfac: i0.ɵɵFactoryDeclaration<WrappedControlSuperclass<any, any>, never>;
    static ɵdir: i0.ɵɵDirectiveDeclaration<WrappedControlSuperclass<any, any>, never, never, {}, {}, never, never, false, never>;
}
